@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike Meyer
-# Copyright (C) 2005 Arthur de Jong
+# Copyright (C) 2005, 2006 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,10 +30,10 @@ items should be changeble from the command line."""
 import urllib
 
 # Current version of webcheck.
-VERSION = "1.9.5"
+VERSION = '1.9.7'
 
 # The homepage of webcheck.
-HOMEPAGE = "http://ch.tudelft.nl/~arthur/webcheck/"
+HOMEPAGE = 'http://ch.tudelft.nl/~arthur/webcheck/'
 
 # Whether to consider any URL not starting with the base URL to be external.
 # This is the state of the -b command line option.
@@ -46,8 +46,15 @@ AVOID_EXTERNAL_LINKS = False
 # The proxy configuration.
 PROXIES = urllib.getproxies_environment()
 
+# IO timeout as passed to socket.setdefaulttimeout()
+# value is a float in seconds None disables the timeout
+IOTIMEOUT = 10.0
+
 # Output directory. This is the state of the -o command line option.
-OUTPUT_DIR = "."
+OUTPUT_DIR = '.'
+
+# Whether to try to read a state file to continue from.
+CONTINUE = False
 
 # This is the time in seconds to wait between requests. This is the state of
 # the -w command line option.
@@ -58,18 +65,19 @@ WAIT_BETWEEN_REQUESTS = 0
 REDIRECT_DEPTH = 5
 
 # The list of plugins that will be used to generate the report.
-PLUGINS = ["sitemap",
-           "urllist",
-           "images",
-           "external",
-           "notchkd",
-           "badlinks",
-           "old",
-           "new",
-           "slow",
-           "notitles",
-           "problems",
-           "about"]
+PLUGINS = [ 'anchors',
+            'sitemap',
+            'urllist',
+            'images',
+            'external',
+            'notchkd',
+            'badlinks',
+            'old',
+            'new',
+            'size',
+            'notitles',
+            'problems',
+            'about' ]
 
 # Whether to overwrite files without asking. This is the state of the -f
 # command line option.
