@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike W. Meyer
-# Copyright (C) 2005 Arthur de Jong
+# Copyright (C) 2005, 2006 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,26 +38,56 @@ import re
 import time
 
 # pattern for matching spaces
-_spacepattern = re.compile(" ")
+_spacepattern = re.compile(' ')
+
+# pattern for matching url encoded characters
+_urlencpattern = re.compile('(%[0-9a-fA-F]{2})', re.IGNORECASE)
+
+# pattern to match anchor part of a url
+_anchorpattern = re.compile('#([^#]+)$')
+
+# characters that should not be escaped in urls
+_reservedurlchars = ';/?:@&=+$,%#'
+_okurlchars = '-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ' \
+              '_abcdefghijklmnopqrstuvwxyz~'
+
+def urlescape(url):
+    """Ensure that escaping in the url is consistent."""
+    # url decode any printable normal characters
+    # except reserved characters with special meanings in urls
+    for c in _urlencpattern.findall(url):
+        r = chr(int(c[1:3], 16))
+        if r in _okurlchars:
+            url = url.replace(c, r)
+    # url encode any nonprintable or problematic characters
+    # (but not reserved chars)
+    url = ''.join(
+      [ (x not in _reservedurlchars and
+         x not in _okurlchars) and ('%%%02X' % ord(x))
+        or x
+        for x in url ] )
+    return url
 
 def _urlclean(url):
     """Clean the url of uneccesary parts."""
-    # split the url in useful parts (discarding fragment)
-    (scheme, netloc, path, query) = urlparse.urlsplit(url)[0:4]
-    if ( scheme == "http" or scheme == "https" or scheme == "ftp" ):
+    # make escaping consistent
+    url = urlescape(url)
+    # split the url in useful parts
+    (scheme, netloc, path, query) = urlparse.urlsplit(url)[:4]
+    if ( scheme == 'http' or scheme == 'https' or scheme == 'ftp' ):
         # http(s) urls should have a non-empty path
-        if path == "":
-            path="/"
+        if path == '':
+            path = '/'
         # make hostname lower case
         (userpass, hostport) = urllib.splituser(netloc)
-        netloc=hostport.lower()
+        netloc = hostport.lower()
         # trim trailing :
-        if netloc[-1:] == ":":
+        if netloc[-1:] == ':':
             netloc = netloc[:-1]
         if userpass is not None:
-            netloc = userpass+"@"+netloc
-    # put the url back together again
-    return urlparse.urlunsplit((scheme, netloc, path, query, ""))
+            netloc = userpass+'@'+netloc
+    # put the url back together again (discarding fragment)
+    return urlparse.urlunsplit((scheme, netloc, path, query, ''))
 
 class Site:
     """Class to represent gathered data of a site.
@@ -74,45 +104,47 @@ class Site:
         # list of internal urls
         self._internal_urls = []
         # list of regexps considered internal
-        self._internal_res = []
+        self._internal_res = {}
         # list of regexps considered external
-        self._external_res = []
+        self._external_res = {}
         # list of regexps matching links that should not be checked
-        self._yanked_res = []
+        self._yanked_res = {}
         # map of scheme+netloc to robot handleds
         self._robotparsers = {}
         # a map of urls to Link objects
         self.linkMap = {}
+        # base url that can be used as start of site
+        self.base = None
 
-    def add_internal(self,url):
+    def add_internal(self, url):
         """Add the given url and consider all urls below it to be internal.
         These links are all marked for checking with the crawl() function."""
-        url=_urlclean(url)
+        url = _urlclean(url)
         if url not in self._internal_urls:
             self._internal_urls.append(url)
 
-    def add_internal_re(self,exp):
+    def add_internal_re(self, exp):
         """Adds the gived regular expression as a pattern to match internal
         urls."""
-        self._internal_res.append(re.compile(exp,re.IGNORECASE))
+        self._internal_res[exp] = re.compile(exp, re.IGNORECASE)
 
-    def add_external_re(self,exp):
+    def add_external_re(self, exp):
         """Adds the gived regular expression as a pattern to match external
         urls."""
-        self._external_res.append(re.compile(exp,re.IGNORECASE))
+        self._external_res[exp] = re.compile(exp, re.IGNORECASE)
 
-    def add_yanked_re(self,exp):
+    def add_yanked_re(self, exp):
         """Adds the gived regular expression as a pattern to match urls that
         will not be checked at all."""
-        self._yanked_res.append(re.compile(exp,re.IGNORECASE))
+        self._yanked_res[exp] = re.compile(exp, re.IGNORECASE)
 
-    def _is_internal(self,link):
+    def _is_internal(self, link):
         """Check whether the specified url is external or internal.
         This uses the urls marked with add_internal() and the regular
         expressions passed with add_external_re()."""
         # check if it is internal through the regexps
-        for x in self._internal_res:
-            if x.search(link.url) is not None:
+        for regexp in self._internal_res.values():
+            if regexp.search(link.url) is not None:
                 return True
         res = False
         # check that the url starts with an internal url
@@ -128,50 +160,52 @@ class Site:
         if not res:
             return False
         # check if it is external through the regexps
-        for x in self._external_res:
+        for x in self._external_res.values():
             # if the url matches it is external and we can stop
             if x.search(link.url) is not None:
                 return False
         return True
 
-    def _get_robotparser(self,link):
+    def _get_robotparser(self, link):
         """Return the proper robots parser for the given url or None if one
         cannot be constructed. Robot parsers are cached per scheme and
         netloc."""
         # only some schemes have a meaningful robots.txt file
-        if link.scheme != "http" and link.scheme != "https":
-            debugio.debug("crawler._get_robotparser() called with unsupported scheme (%s)" % link.scheme)
+        if link.scheme != 'http' and link.scheme != 'https':
+            debugio.debug('crawler._get_robotparser() called with unsupported scheme (%s)' % link.scheme)
             return None
         # split out the key part of the url
-        location = urlparse.urlunsplit((link.scheme, link.netloc, "", "", ""))
+        location = urlparse.urlunsplit((link.scheme, link.netloc, '', '', ''))
         # try to create a new robotparser if we don't already have one
         if not self._robotparsers.has_key(location):
             import httplib
-            debugio.info("  getting robots.txt for %s" % location)
+            debugio.info('  getting robots.txt for %s' % location)
             self._robotparsers[location] = None
             try:
-                rp=robotparser.RobotFileParser()
-                rp.set_url(urlparse.urlunsplit((link.scheme, link.netloc, "/robots.txt", "", "")))
+                rp = robotparser.RobotFileParser()
+                rp.set_url(urlparse.urlunsplit(
+                  (link.scheme, link.netloc, '/robots.txt', '', '') ))
                 rp.read()
                 self._robotparsers[location] = rp
             except (TypeError, IOError, httplib.HTTPException):
+                # ignore any problems setting up robot parser
                 pass
         return self._robotparsers[location]
 
-    def _is_yanked(self,link):
+    def _is_yanked(self, link):
         """Check whether the specified url should not be checked at all.
         This uses the regualr expressions passed with add_yanked_re() and the
         robots information present."""
         # check if it is yanked through the regexps
-        for x in self._yanked_res:
+        for regexp in self._yanked_res.values():
             # if the url matches it is yanked and we can stop
-            if x.search(link.url) is not None:
-                return "yanked"
+            if regexp.search(link.url) is not None:
+                return 'yanked'
         # check if we should avoid external links
         if not link.isinternal and config.AVOID_EXTERNAL_LINKS:
-            return "external avoided"
+            return 'external avoided'
         # skip schemes not haveing robot.txt files
-        if link.scheme != "http" and link.scheme != "https":
+        if link.scheme != 'http' and link.scheme != 'https':
             return False
         # skip robot checks for external urls
         # TODO: make this configurable
@@ -179,12 +213,12 @@ class Site:
             return False
         # check robots for remaining links
         rp = self._get_robotparser(link)
-        if rp is not None and not rp.can_fetch('webcheck',link.url):
-            return "robot restriced"
+        if rp is not None and not rp.can_fetch('webcheck', link.url):
+            return 'robot restriced'
         # fall back to allowing the url
         return False
 
-    def _get_link(self,url):
+    def get_link(self, url):
         """Return a link object for the given url.
         This function checks the map of cached link objects for an
         instance."""
@@ -194,21 +228,29 @@ class Site:
         if self.linkMap.has_key(url):
             return self.linkMap[url]
         # create a new instance
-        return Link(self,url)
+        return Link(self, url)
 
-    def crawl(self):
+    def crawl(self, serfp=None):
         """Crawl the website based on the urls specified with
-        add_internal()."""
+        add_internal(). If the serialization file pointer
+        is specified the crawler writes out updated links to
+        the file while crawling the site."""
         # TODO: have some different scheme to crawl a site (e.g. separate
         #       internal and external queues, threading, etc)
         tocheck = []
-        for u in self._internal_urls:
-            tocheck.append(self._get_link(u))
+        # add all unfetched site urls
+        for link in self.linkMap.values():
+            if not link.isyanked and not link.isfetched:
+                tocheck.append(link)
+        # add all internal urls
+        for url in self._internal_urls:
+            tocheck.append(self.get_link(url))
         # repeat until we have nothing more to check
+        fetchedlinks = 0
         while len(tocheck) > 0:
-            debugio.debug("crawler.crawl(): items left to check: %d" % len(tocheck))
+            debugio.debug('crawler.crawl(): items left to check: %d' % len(tocheck))
             # choose a link from the tocheck list
-            link=tocheck.pop(0)
+            link = tocheck.pop(0)
             # skip link it there is nothing to check
             if link.isyanked or link.isfetched:
                 continue
@@ -222,20 +264,48 @@ class Site:
             for embed in link.embedded:
                 if not embed.isyanked and not embed.isfetched and not embed in tocheck:
                     tocheck.append(embed)
+            # serialize all as of yet unserialized links
+            fetchedlinks += 1
+            # TODO: make this configurable
+            if serfp and fetchedlinks >= 5:
+                fetchedlinks = 0
+                import serialize
+                for link in self.linkMap.values():
+                    if link._ischanged:
+                        serialize.serialize_link(serfp, link)
+                        link._ischanged = False
+                serfp.flush()
             # sleep between requests if configured
             if config.WAIT_BETWEEN_REQUESTS > 0:
-                debugio.debug('sleeping %s seconds' %  config.WAIT_BETWEEN_REQUESTS)
+                debugio.debug('crawler.crawl(): sleeping %s seconds' % config.WAIT_BETWEEN_REQUESTS)
                 time.sleep(config.WAIT_BETWEEN_REQUESTS)
+        # serialize remaining changed links
+        if serfp:
+            import serialize
+            for link in self.linkMap.values():
+                if link._ischanged:
+                    serialize.serialize_link(serfp, link)
+                    link._ischanged = False
+            serfp.flush()
+
+    def postprocess(self):
         # build the list of urls that were set up with add_internal() that
         # do not have a parent (they form the base for the site)
-        bases = [ ]
-        for u in self._internal_urls:
-            l = self.linkMap[u].follow_link(True)
-            if l == None:
+        bases = []
+        for url in self._internal_urls:
+            link = self.linkMap[url].follow_link()
+            if link == None:
+                debugio.warn('base link %s redirects to nowhere' % url)
                 continue
-            # if the link has no parent add it to the result list unless it is the first one
-            if len(l.parents) == 0 or len(bases) == 0:
-                bases.append(l)
+            # if the link has no parent add it to the result list,
+            # unless it is the first one
+            if len(link.parents) == 0 or len(bases) == 0:
+                debugio.debug('crawler.postprocess(): adding %s to bases' % link.url)
+                bases.append(link)
+        # if we got no bases, just use the first internal one
+        if len(bases) == 0:
+            debugio.debug('crawler.postprocess(): fallback to adding %s to bases' % self._internal_urls[0])
+            bases.append(self.linkMap[self._internal_urls[0]])
         # do a breadth first traversal of the website to determin depth and
         # figure out page children
         tocheck = []
@@ -244,7 +314,7 @@ class Site:
             tocheck.append(link)
         # repeat until we have nothing more to check
         while len(tocheck) > 0:
-            debugio.debug("crawler.crawl(): items left to examine: %d" % len(tocheck))
+            debugio.debug('crawler.postprocess(): items left to examine: %d' % len(tocheck))
             # choose a link from the tocheck list
             link = tocheck.pop(0)
             # figure out page children
@@ -274,6 +344,8 @@ class Link:
       pagechildren - list of child pages, including children of embedded
                      elements
       embedded   - list of links to embeded content
+      anchors    - list of anchors defined on the page
+      reqanchors - list of anchors requesten for this page anchor->link*
       depth      - the number of clicks from the base urls this page to
                    find
       isinternal - whether the link is considered to be internal
@@ -301,10 +373,12 @@ class Link:
         # store a reference to the site
         self.site = site
         # split the url in useful parts and store the parts
-        (self.scheme, self.netloc, self.path, self.query) = urlparse.urlsplit(url)[0:4]
+        (self.scheme, self.netloc, self.path, self.query) = \
+          urlparse.urlsplit(url)[0:4]
         # store the url (without the fragment)
-        url=urlparse.urlunsplit((self.scheme, self.netloc, self.path, self.query, ""))
-        self.url=url
+        url = urlparse.urlunsplit(
+          (self.scheme, self.netloc, self.path, self.query, '') )
+        self.url = url
         # ensure that we are not creating something that already exists
         assert not self.site.linkMap.has_key(url)
         # store the Link object in the linkMap
@@ -318,6 +392,8 @@ class Link:
         self.children = []
         self.pagechildren = None
         self.embedded = []
+        self.anchors = []
+        self.reqanchors = {}
         self.depth = None
         self.isfetched = False
         self.ispage = False
@@ -331,6 +407,8 @@ class Link:
         self.linkproblems = []
         self.pageproblems = []
         self.redirectdepth = 0
+        self.redirectlist = None
+        self._ischanged = False
 
     def _checkurl(self, url):
         """Check to see if the url is formatted properly, correct formatting
@@ -338,7 +416,18 @@ class Link:
         if _spacepattern.search(url):
             self.add_pageproblem('link contains unescaped spaces: %s' % url)
             # replace spaces by %20
-            url=_spacepattern.sub("%20",url)
+            url = _spacepattern.sub('%20', url)
+        # find anchor part
+        try:
+            # get the anchor
+            anchor = _anchorpattern.search(url).group(1)
+            # get link for url we link to
+            child = self.site.get_link(url)
+            # store anchor
+            child.add_reqanchor(self, anchor)
+        except AttributeError:
+            # ignore problems lookup up anchor
+            pass
         return url
 
     def add_child(self, child):
@@ -349,10 +438,11 @@ class Link:
             return
         # convert the url to a link object if we were called with a url
         if type(child) is str:
-            child = self.site._get_link(self._checkurl(child))
+            child = self.site.get_link(self._checkurl(child))
         # add to children
         if child not in self.children:
             self.children.append(child)
+            self._ischanged = True
         # add self to parents of child
         if self not in child.parents:
             child.parents.append(self)
@@ -364,45 +454,74 @@ class Link:
             return
         # convert the url to a link object if we were called with a url
         if type(link) is str:
-            link = self.site._get_link(self._checkurl(link))
+            link = self.site.get_link(self._checkurl(link))
         # add to embedded
         if link not in self.embedded:
             self.embedded.append(link)
+            self._ischanged = True
         # add self to parents of embed
         if self not in link.parents:
             link.parents.append(self)
 
+    def add_anchor(self, anchor):
+        """Indicate that this page contains the specified anchor."""
+        # add anchor
+        if anchor in self.anchors:
+            self.add_pageproblem(
+              'anchor "%(anchor)s" defined multiple times'
+              % { 'anchor':   anchor })
+        else:
+            self.anchors.append(anchor)
+            self._ischanged = True
+
+    def add_reqanchor(self, parent, anchor):
+        """Indicate that the specified link contains a reference to the
+        specified anchor. This can be checked later."""
+        # convert the url to a link object if we were called with a url
+        if type(parent) is str:
+            parent = self.site.get_link(self._checkurl(parent))
+        # add anchor
+        if anchor in self.reqanchors:
+            if parent not in self.reqanchors[anchor]:
+                self.reqanchors[anchor].append(parent)
+                self._ischanged = True
+        else:
+            self.reqanchors[anchor] = [parent]
+            self._ischanged = True
+
     def redirect(self, url):
         """Indicate that this link redirects to the specified url. Maximum
         redirect counting is done as well as loop detection."""
-        # figure out determin depth
+        # figure out depth and urls that have been visited in this
+        # redirect list
         redirectdepth = 0
         redirectlist = []
-        for p in self.parents:
-            if p.redirectdepth > redirectdepth:
-                redirectdepth = p.redirectdepth
-                redirectlist = p.redirectlist
+        for parent in self.parents:
+            if parent.redirectdepth > redirectdepth:
+                redirectdepth = parent.redirectdepth
+                redirectlist = parent.redirectlist
         self.redirectdepth = redirectdepth + 1
         self.redirectlist = redirectlist
         self.redirectlist.append(self.url)
         # check depth
         if self.redirectdepth >= config.REDIRECT_DEPTH:
-            self.add_linkproblem("too many redirects (%d)" % self.redirectdepth)
+            self.add_linkproblem('too many redirects (%d)' % self.redirectdepth)
             return None
         # check for redirect to self
         url = self._checkurl(url)
         if url == self.url:
-            self.add_linkproblem("redirect same as source: %s" % url)
+            self.add_linkproblem('redirect same as source: %s' % url)
             return None
         # check for redirect loop
         if url in self.redirectlist:
-            self.add_linkproblem("redirect loop %s" % url)
+            self.add_linkproblem('redirect loop %s' % url)
         # add child
         self.add_child(url)
 
     def add_linkproblem(self, problem):
         """Indicate that something went wrong while retreiving this link."""
         self.linkproblems.append(problem)
+        self._ischanged = True
 
     def add_pageproblem(self, problem):
         """Indicate that something went wrong with parsing the document."""
@@ -410,56 +529,55 @@ class Link:
         if not self.isinternal:
             return
         self.pageproblems.append(problem)
+        self._ischanged = True
 
     def fetch(self):
         """Attempt to fetch the url (if isyanked is not True) and fill in link
         attributes (based on isinternal)."""
         # fully ignore links that should not be feteched
         if self.isyanked:
-            debugio.info("  %s" % self.url)
-            debugio.info("    "+self.isyanked)
+            debugio.info('  %s' % self.url)
+            debugio.info('    ' + self.isyanked)
             return
         # see if we can import the proper module for this scheme
         schememodule = schemes.get_schememodule(self.scheme)
         if schememodule is None:
-            self.isyanked="unsupported scheme ("+self.scheme+")"
-            debugio.info("  %s" % self.url)
-            debugio.info("    "+self.isyanked)
+            self.isyanked = 'unsupported scheme (' + self.scheme + ')'
+            self._ischanged = True
+            debugio.info('  %s' % self.url)
+            debugio.info('    ' + self.isyanked)
             return
-        debugio.info("  %s" % self.url)
-        content=schememodule.fetch(self, parsers.get_mimetypes())
+        debugio.info('  %s' % self.url)
+        content = schememodule.fetch(self, parsers.get_mimetypes())
         self.isfetched = True
+        self._ischanged = True
         # skip parsing of content if we were returned nothing
         if content is None:
             return
         # find a parser for the content-type
         parsermodule = parsers.get_parsermodule(self.mimetype)
         if parsermodule is None:
-            debugio.debug("crawler.Link.fetch(): unsupported content-type: %s" % self.mimetype)
+            debugio.debug('crawler.Link.fetch(): unsupported content-type: %s' % self.mimetype)
             return
         # parse the content
         parsermodule.parse(content, self)
 
-    def follow_link(self, delifunref=False, visited=[]):
+    def follow_link(self, visited=None):
         """If this link represents a redirect return the redirect target,
-        otherwise return self. If delifunref is set this link is discarded
-        if it has no parents. If this redirect does not find a referenced
+        otherwise return self. If this redirect does not find a referenced
         link None is returned."""
         if self.redirectdepth == 0:
             return self
         if len(self.children) == 0:
             return None
+        # set up visited
+        if visited is None:
+            visited = []
         # check for loops
         visited.append(self)
         if self.children[0] in visited:
             return None
-        # remove link if this is the only place that it's used
-        if (len(self.parents) == 0) and delifunref:
-           # remove me from the linkMap
-           del self.site.linkMap[self.url]
-           # remove me from parents of child
-           self.children[0].parents.remove(self)
-        return self.children[0].follow_link(delifunref, visited)
+        return self.children[0].follow_link(visited)
 
     def _pagechildren(self):
         """Determin the page children of this link, combining the children of
@@ -471,7 +589,7 @@ class Link:
         # add my own children, following redirects
         for child in self.children:
             # follow redirects
-            child=child.follow_link()
+            child = child.follow_link()
             # skip children we already have
             if child is None or child in self.pagechildren:
                 continue
