@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike W. Meyer
-# Copyright (C) 2005 Arthur de Jong
+# Copyright (C) 2005, 2006 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,23 +26,27 @@
 
 __title__ = 'bad links'
 __author__ = 'Arthur de Jong'
+__outputfile__ = 'badlinks.html'
 
-import config
 import plugins
 
-def generate(fp,site):
+def generate(site):
     """Present the list of bad links to the given file descriptor."""
     # find all links with link problems
-    links = filter(lambda a: len(a.linkproblems)>0, site.linkMap.values())
+    links = [ x
+              for x in site.linkMap.values()
+              if len(x.linkproblems)>0 ]
     # sort list
     links.sort(lambda a, b: cmp(a.url, b.url))
     # present results
+    fp = plugins.open_html(plugins.badlinks, site)
     if not links:
         fp.write(
           '   <p class="description">\n'
           '    There were no problems retrieving links from the website.\n'
           '   </p>\n'
           '   <ol>\n' )
+        plugins.close_html(fp)
         return
     fp.write(
       '   <p class="description">\n'
@@ -60,17 +64,18 @@ def generate(fp,site):
         for problem in link.linkproblems:
             fp.write(
               '      <li>%(problem)s</li>\n'
-              % { 'problem':  plugins.escape(problem) })
+              % { 'problem':  plugins.htmlescape(problem) })
         fp.write(
           '     </ul>\n')
         # present a list of parents
         link.parents.sort()
-        plugins.print_parents(fp,link,'     ')
+        plugins.print_parents(fp, link, '     ')
         # add a reference to the problem map
         for problem in link.linkproblems:
             for parent in link.parents:
-                parent.add_pageproblem("bad link: " + link.url + ": " + problem)
+                parent.add_pageproblem('bad link: ' + link.url + ': ' + problem)
         fp.write(
           '    </li>\n')
     fp.write(
       '   </ol>\n' )
+    plugins.close_html(fp)
