@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike W. Meyer
-# Copyright (C) 2005 Arthur de Jong
+# Copyright (C) 2005, 2006 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,11 +26,12 @@
 
 __title__ = 'problems by author'
 __author__ = 'Arthur de Jong'
+__outputfile__ = 'problems.html'
 
 import plugins
 import urllib
 
-def generate(fp,site):
+def generate(site):
     """Output the overview of problems to the given file descriptor."""
     # make a list of problems per author
     problem_db = {}
@@ -48,11 +49,13 @@ def generate(fp,site):
             problem_db[author].append(link)
         else:
             problem_db[author] = [link]
+    fp = plugins.open_html(plugins.problems, site)
     if not problem_db:
         fp.write(
           '   <p class="description">\n'
           '    No problems were found on this site, hurray.\n'
           '   </p>\n' )
+        plugins.close_html(fp)
         return
     # print description
     fp.write(
@@ -61,7 +64,7 @@ def generate(fp,site):
       '    author.\n'
       '   </p>\n' )
     # get a list of authors
-    authors=problem_db.keys()
+    authors = problem_db.keys()
     authors.sort()
     # generate short list of authors
     if len(authors) > 1:
@@ -70,7 +73,7 @@ def generate(fp,site):
             fp.write(
               '    <li><a href="#%(authorref)s">Author: %(author)s</a></li>\n'
               % { 'authorref': urllib.quote(author,''),
-                  'author':    plugins.escape(author) })
+                  'author':    plugins.htmlescape(author) })
         fp.write('   </ul>\n')
     # generate problem report
     fp.write('   <ul>\n')
@@ -80,7 +83,7 @@ def generate(fp,site):
           '      <a name="%(authorref)s">Author: %(author)s</a>\n'
           '      <ul>\n'
           % { 'authorref': urllib.quote(author,''),
-              'author':    plugins.escape(author) })
+              'author':    plugins.htmlescape(author) })
         # sort pages by url
         problem_db[author].sort(lambda a, b: cmp(a.url, b.url))
         # list problems for this author
@@ -97,7 +100,7 @@ def generate(fp,site):
             for problem in link.pageproblems:
                 fp.write(
                   '      <li>%(problem)s</li>\n'
-                  % { 'problem':  plugins.escape(problem) })
+                  % { 'problem':  plugins.htmlescape(problem) })
             # end the list item
             fp.write(
               '     </ul>\n'
@@ -107,3 +110,4 @@ def generate(fp,site):
           '     </li>\n' )
     fp.write(
       '   </ul>\n' )
+    plugins.close_html(fp)
